﻿using System.Collections.Generic;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media.Animation;

namespace TestProject
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 커스텀 완화 함수 리스트 - CustomEasingFunctionList

        /// <summary>
        /// 커스텀 완화 함수 리스트
        /// </summary>
        private List<CustomEasingFunction> CustomEasingFunctionList { get; } = new List<CustomEasingFunction>()
        {
            new CustomEasingFunction("BackEase"       , new BackEase()       ),
            new CustomEasingFunction("BounceEase"     , new BounceEase()     ),
            new CustomEasingFunction("CircleEase"     , new CircleEase()     ),
            new CustomEasingFunction("CubicEase"      , new CubicEase()      ),
            new CustomEasingFunction("ElasticEase"    , new ElasticEase()    ),
            new CustomEasingFunction("ExponentialEase", new ExponentialEase()),
            new CustomEasingFunction("PowerEase"      , new PowerEase()      ),
            new CustomEasingFunction("QuadraticEase"  , new QuadraticEase()  ),
            new CustomEasingFunction("QuarticEase"    , new QuarticEase()    ),
            new CustomEasingFunction("QuinticEase"    , new QuinticEase()    ),
            new CustomEasingFunction("SineEase"       , new SineEase()       )
        };

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            #region 윈도우 크기를 설정한다.

            double width  = 800d;
            double height = 600d;

            double dpi = (double)DisplayInformation.GetForCurrentView().LogicalDpi;

            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;

            Size windowSize = new Size(width * 96d / dpi, height * 96d / dpi);

            ApplicationView.PreferredLaunchViewSize = windowSize;

            Window.Current.Activate();

            ApplicationView.GetForCurrentView().TryResizeView(windowSize);

            #endregion
            #region 윈도우 제목을 설정한다.

            ApplicationView.GetForCurrentView().Title = "DoubleAnimation 클래스 : EasingFunction 속성 사용하기";

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 애니메이션 버튼 클릭시 처리하기 - animateButton_Click(sender, e)

        /// <summary>
        /// 애니메이션 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void animateButton_Click(object sender, RoutedEventArgs e)
        {
            EasingFunctionBase easingFunctionBase = this.easingModeComboBox.SelectedValue as EasingFunctionBase;

            easingFunctionBase.EasingMode = GetEasingMode();

            (this.storyboard.Children[0] as DoubleAnimation).EasingFunction = easingFunctionBase;

            this.storyboard.Children[0].SetValue(DoubleAnimation.FromProperty, this.translationTransform.X              );
            this.storyboard.Children[0].SetValue(DoubleAnimation.ToProperty  , this.translationTransform.X > 0 ? 0 : 200);

            this.storyboard.Begin();
        }

        #endregion
        #region 완화 모드 콤보 박스 로드시 처리하기 - easingModeComboBox_Loaded(sender, e)

        /// <summary>
        /// 완화 모드 콤보 박스 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void easingModeComboBox_Loaded(object sender, RoutedEventArgs e)
        {
            this.easingModeComboBox.SelectedIndex = 0;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 완화 모드 구하기 - GetEasingMode()

        /// <summary>
        /// 완화 모드 구하기
        /// </summary>
        /// <returns>완화 모드</returns>
        private EasingMode GetEasingMode()
        {
            if(this.easeOutRadioButton.IsChecked == true)
            {
                return EasingMode.EaseOut;
            }
            else if(this.easeInRadioButton.IsChecked == true)
            {
                return EasingMode.EaseIn;
            }
            else
            {
                return EasingMode.EaseInOut;
            }
        }

        #endregion
    }
}